package com.ruoyi.system.service.impl;

import org.springframework.transaction.annotation.Transactional;
import com.ruoyi.system.api.domain.SysUser;
import com.ruoyi.system.enums.BussLogEnum;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.domain.SnTemplate;
import java.util.Collection;
import java.util.ArrayList;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.function.Function;
import com.ruoyi.common.core.web.domain.AjaxResult;
import com.ruoyi.common.core.utils.DateUtils;
import java.util.List;
import com.ruoyi.system.domain.SnTemplatePerms;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.ISnTemplateService;
import com.ruoyi.system.service.ISysBussLogService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SnTemplatePermsMapper;
import org.springframework.stereotype.Service;
import com.ruoyi.system.service.ISnTemplatePermsService;

@Service
public class SnTemplatePermsServiceImpl implements ISnTemplatePermsService
{
    @Autowired
    private SnTemplatePermsMapper snTemplatePermsMapper;
    @Autowired
    private ISysBussLogService sysBussLogService;
    @Autowired
    private ISnTemplateService snTemplateService;
    @Autowired
    private SysUserMapper userMapper;
    
    public SnTemplatePerms selectSnTemplatePermsById(final Long id) {
        return this.snTemplatePermsMapper.selectSnTemplatePermsById(id);
    }
    
    public List<SnTemplatePerms> selectSnTemplatePermsList(final SnTemplatePerms snTemplatePerms) {
        return this.snTemplatePermsMapper.selectSnTemplatePermsList(snTemplatePerms);
    }
    
    public int insertSnTemplatePerms(final SnTemplatePerms snTemplatePerms) {
        snTemplatePerms.setCreateTime(DateUtils.getNowDate());
        return this.snTemplatePermsMapper.insertSnTemplatePerms(snTemplatePerms);
    }
    
    public int updateSnTemplatePerms(final SnTemplatePerms snTemplatePerms) {
        return this.snTemplatePermsMapper.updateSnTemplatePerms(snTemplatePerms);
    }
    
    @Transactional
    public AjaxResult updateUserPerms(final List<Long> templates, final Long userId) {
        final SnTemplatePerms params = new SnTemplatePerms();
        params.setUserId(userId);
        final List<SnTemplatePerms> olds = this.snTemplatePermsMapper.selectSnTemplatePermsList(params);
        final List<Long> oldIds = olds.stream().map((Function<? super Object, ?>)SnTemplatePerms::getTemplateId).distinct().collect((Collector<? super Object, ?, List<Long>>)Collectors.toList());
        this.snTemplatePermsMapper.deletePermsByUserId(userId);
        if (!templates.isEmpty()) {
            final List<SnTemplatePerms> lists = templates.stream().map(templateId -> {
                final SnTemplatePerms obj = new SnTemplatePerms();
                obj.setTemplateId(templateId);
                obj.setUserId(userId);
                obj.setCreateTime(DateUtils.getNowDate());
                return obj;
            }).collect((Collector<? super Object, ?, List<SnTemplatePerms>>)Collectors.toList());
            this.snTemplatePermsMapper.insertBatch((List)lists);
        }
        try {
            final List<Long> reduceIds = oldIds.stream().filter(id -> !templates.contains(id)).collect((Collector<? super Object, ?, List<Long>>)Collectors.toList());
            final List<Long> addIds = templates.stream().filter(id -> !oldIds.contains(id)).collect((Collector<? super Object, ?, List<Long>>)Collectors.toList());
            final List<Long> tmpIds = new ArrayList<Long>();
            tmpIds.addAll(reduceIds);
            tmpIds.addAll(addIds);
            final SnTemplate snTemplate = new SnTemplate();
            snTemplate.getParams().put("templateIds", tmpIds.stream().distinct().toArray(Long[]::new));
            final List<SnTemplate> tmpList = this.snTemplateService.selectSnTemplateList(snTemplate);
            final String reduceStr = tmpList.stream().filter(tp -> reduceIds.contains(tp.getTemplateId())).map(tp -> "【" + tp.getName() + "】").collect((Collector<? super Object, ?, String>)Collectors.joining("\n"));
            final String addStr = tmpList.stream().filter(tp -> addIds.contains(tp.getTemplateId())).map(tp -> "【" + tp.getName() + "】").collect((Collector<? super Object, ?, String>)Collectors.joining("\n"));
            String remark = "";
            if (StringUtils.isNotEmpty(reduceStr)) {
                remark = remark + "删除模板\n" + reduceStr + "\n\n\n";
            }
            if (StringUtils.isNotEmpty(addStr)) {
                remark = remark + "增加模板\n" + addStr + "\n";
            }
            if (StringUtils.isNotEmpty(remark)) {
                final SysUser user = this.userMapper.selectUserById(userId);
                this.sysBussLogService.insertSysBussLog(BussLogEnum.OptTemplate.getType(), user.getUserId(), user.getUserName(), remark);
            }
        }
        catch (final Exception ex) {}
        return AjaxResult.success();
    }
    
    public int deleteSnTemplatePermsByIds(final Long[] ids) {
        return this.snTemplatePermsMapper.deleteSnTemplatePermsByIds(ids);
    }
    
    public int deleteSnTemplatePermsById(final Long id) {
        return this.snTemplatePermsMapper.deleteSnTemplatePermsById(id);
    }
}
